const path = require('path');
const {babel} = require('@rollup/plugin-babel');
const postcss = require('rollup-plugin-postcss');
const sass = require('node-sass');
const image = require('@rollup/plugin-image');
const svg = require('rollup-plugin-svg');
const url = require('rollup-plugin-url');
const scss = require('rollup-plugin-scss');
const css = require('rollup-plugin-css-only');
// rollup.config.js
const vue = require("rollup-plugin-vue");
const nodeResolve = require("rollup-plugin-node-resolve");
const commonjs = require("rollup-plugin-commonjs");
const terser = require("rollup-plugin-terser");
const requireContext = require("rollup-plugin-require-context");
const peerDepsExternal = require("rollup-plugin-peer-deps-external");


const resolveFile = function (filePath) {
    return path.join(__dirname, '..', filePath)
}

const isProductionEnv = process.env.NODE_ENV === 'production'

const babelOptions = {
    "presets": ['@babel/preset-env'],
    "exclude": "node_modules/**"
}

const processSass = function (context, payload) {

    return new Promise((resolve, reject) => {
        sass.render({
            file: context
        }, function (err, result) {
            if (!err) {
                resolve(result);
            } else {
                reject(err)
            }
        });
    })
}

module.exports = [
    {

        input: resolveFile('src/index.js'),
        external: ['vue'],
        output: {
            file: resolveFile('dist/index.js'),
            format: 'umd',
            name: 'bundle',
            globals: {vue: 'Vue'},

        },
        plugins: [
            nodeResolve({jsnext: true}),
            css({ output: 'dist/main.css' }),
            vue({
                css: true,
                compileTemplate: true,
            }),
            babel(babelOptions),

            commonjs(),

            scss(),
            postcss({
                extract: true,
                minimize: isProductionEnv,
                extensions: ['.css', '.scss'],
                process: processSass,

            }),
            url({
                include: ['**/*.woff', '**/*.ttf'], // 打包字体为base64
                limit: Infinity
            }),
            image(),
            svg(),
            requireContext(),
            terser.terser(),
            peerDepsExternal({
                includeDependencies: !isProductionEnv
            })
        ],

    },
]