import {concatHtmlWithTagAndAttrs, isBind, keyQuote, name, quote, randomWord, uid, value, valueQuote} from "./utils";

const tc = Symbol.for("tableCol")
const tr = Symbol.for("tableRow")

class Table {
    constructor(tableId, htmlOfTable, elData, columnsData) {
        this.tableId = tableId
        this.htmlOfTable = htmlOfTable
        this.elData = elData
        this.elDataName = tableId + 'Data'
        this.columnsData = columnsData
        this.columnsDataName = tableId + 'ColumnName'
    }

}

Table.prototype.getTableHtml = function () {
    return this.htmlOfTable
}
Table.prototype.getTableData = function () {
    return {
        dataName: this.elDataName,
        data: this.elData,
        columnsDataName: this.columnsDataName,
        columnsData: this.columnsData
    }
}


class VETableBuilder {
    constructor() {
        this.tableId = 'Table' + uid()
        this.columns = []
        this.tableHead = []
        this.columnsData = []
        this.columnsDataName = this.tableId + 'ColumnName'
        this.elData = []
        this.elDataName = this.tableId + 'Data'
    }

}

VETableBuilder.prototype.reset = function () {
    this.tableId = 'Table' + uid()
    this.columns = []
    this.tableHead = []
    this.columnsData = []
    this.columnsDataName = this.tableId + 'ColumnName'
    this.elData = []
    this.elDataName = this.tableId + 'Data'

}
VETableBuilder.prototype.setTableColumn = function (columnName, columnLabel, opt) {

    let column = {
        field: columnName, title: columnLabel, key: randomWord(true,1,4)
    }
    // let prop = Object.create(null)
    // prop[name] = 'prop'
    // prop[value] = columnName
    // prop[keyQuote] = ''
    // prop[valueQuote] = quote
    // prop[isBind] = false
    //
    // let label = Object.create(null)
    // label[name] = 'label'
    // label[value] = columnLabel
    // label[keyQuote] = ''
    // label[valueQuote] = quote
    // label[isBind] = false
    // let elColAttr = [
    //     prop, label
    // ]
    this.columnsData.push(column)

}

VETableBuilder.prototype.setTableHead = function () {
    let data = Object.create(null)
    data[name] = 'table-data'
    data[value] = this.elDataName
    data[keyQuote] = ''
    data[valueQuote] = quote
    data[isBind] = true

    let columns = Object.create(null)
    columns[name] = 'columns'
    columns[value] = this.columnsDataName
    columns[keyQuote] = ''
    columns[valueQuote] = quote
    columns[isBind] = true

    let style = Object.create(null)
    style[name] = 'style'
    style[value] = 'width: 100%'
    style[keyQuote] = ''
    style[valueQuote] = quote
    style[isBind] = false

    let elColAttr = [
        data, style,columns

    ]
    this.tableHead.push(elColAttr)
}

VETableBuilder.prototype.setTableBody = function (columnName, columnLabel, rowValue, rowFormat, newline, opt) {

}

VETableBuilder.prototype.setTableData = function (rows) {
    this.elData = rows
}

VETableBuilder.prototype.generateTableHtml = function () {
    this.setTableHead()
    // let colOfhtmls = []
    // for (let colIndex in  this.columns) {
    //     let colHtml = concatHtmlWithTagAndAttrs('el-table-column', this.columns[colIndex])
    //     colOfhtmls.push(colHtml)
    // }
    let tableHtml = concatHtmlWithTagAndAttrs('ve-table', this.tableHead, '', false)
    let table = new Table(this.tableId, tableHtml, this.elData, this.columnsData)
    return table
}

class TableFactory {
    constructor(opts) {
        this.parent = opts.$this
        this.tableRows = []
        this.tableColumns = []
        this.cache = {};
        if (opts && opts.tablebuilder) {
            this.tablebuilder = opts.tablebuilder
        } else {
            this.tablebuilder = new VETableBuilder()
        }
    }

}

TableFactory.prototype.reset = function () {
    this.tableRows = []
    this.tableColumns = []
    this.cache = {};
    this.tablebuilder.reset()

}
TableFactory.prototype.checkColCountEqualRowCol = function () {
    if (this.tableRows && this.tableRows.length > 0 && this.tableColumns && this.tableColumns.length === Object.keys(this.tableRows[0]).length) {
        return true
    } else if (!this.tableRows) {
        return true
    }
    return false
}

TableFactory.prototype.getTableHtml = function () {
    let table = '';
    if (this.checkColCountEqualRowCol()) {
        table = this.tablebuilder.generateTableHtml()
    } else {
        console.error("table rows count not equal table column count")
    }
    this.reset()
    this.parent.tables.push(table)
    return table.getTableHtml()
}

TableFactory.prototype.pushRow = function () {
    let currentRow = this.getCache(tr)
    if (currentRow) {
        this.pushTableRow()
    } else {
        this.pushTableColumn()
    }
}

TableFactory.prototype.pushTableColumn = function () {
    let colNames = this.clearCacheAndReturn(tc)
    if (colNames) {
        this.tableColumns = colNames
    }
}
TableFactory.prototype.pushTableRow = function () {
    let row = this.clearCacheAndReturn(tr)
    if (row) {
        this.tableRows.push(row)
        this.tablebuilder.setTableData(this.tableRows)

    }
}
TableFactory.prototype.setTableColumn = function (colName) {
    let randomName = randomWord(true, 4, 5)
    this.setCache(tc, randomName, true)
    this.tablebuilder.setTableColumn(randomName, colName, true)
}
TableFactory.prototype.setTableRow = function (row) {
    let colNames = this.tableColumns
    let currentRow = this.getCache(tr)
    if (!currentRow) {
        currentRow = {}
    }
    let colIndex = 0
    if (currentRow && Object.keys(currentRow).length > 0) {
        colIndex = Object.keys(currentRow).length
    }
    if (colNames && colNames.length > colIndex) {
        let currentColName = colNames[colIndex]
        currentRow[currentColName] = row
    }
    this.setCache(tr, currentRow, false)
}
TableFactory.prototype.getCache = function (cacheKey) {
    let key;
    if (typeof cacheKey !== 'symbol' && typeof cacheKey === 'string') {
        key = Symbol.for(cacheKey)
        if (!Symbol.keyFor(key)) {
            return
        }
    } else if (typeof cacheKey === 'symbol') {
        key = cacheKey
    }
    return this.cache[key]
}
TableFactory.prototype.setCache = function (cacheKey, cacheValue, append) {
    let key;
    if (typeof cacheKey !== 'symbol' && typeof cacheKey === 'string') {
        key = Symbol.for(cacheKey)
        if (!Symbol.keyFor(key)) {
            return
        }
    } else if (typeof cacheKey === 'symbol') {
        key = cacheKey
    }

    let cache = this.cache[key]

    if (cache && append) {
        cache.push(cacheValue)
    } else if (cache && !append) {
        this.cache[key] = cache
    } else if (append) {
        cache = [cacheValue]
        this.cache[key] = cache
    } else if (!append) {
        this.cache[key] = cacheValue
    }
}

TableFactory.prototype.clearCacheAndReturn = function (cacheName) {
    let key;
    if (typeof cacheName !== 'symbol' && typeof cacheName === 'string') {
        key = Symbol.for(cacheName)
        if (!Symbol.keyFor(key)) {
            return
        }
    } else if (typeof cacheName === 'symbol') {
        key = cacheName
    }

    let cache = this.cache[key]
    if (cache) {
        delete this.cache[key]
    }
    return cache
}


export {TableFactory}
